###########################################################################
# Copyright (C) 2008 by Wolfgang Otto                      
# All Rights Reserved.                                     
#  
# Permission to use, copy, modify, and distribute this
# software and its documentation for NON-COMMERCIAL purposes
# and without fee is hereby granted provided that this     
# copyright notice appears in all copies.                     
#                                                             
# THE AUTHOR AND PUBLISHE MAKE NO REPRESENTATIONS OR          
# WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER    
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE        
# IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A        
# PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHORS        
# AND PUBLISHER SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED  
# BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING 
# THIS SOFTWARE OR ITS DERIVATIVES.                           
#                                                             
###########################################################################

package Error;

use strict;
use warnings;

use FileHandle;

###########################################################################
# methods
###########################################################################

sub warning {
  my $message = '';
  my $detail = 0;
  my $break = 0;

  if (@_ > 0) {
    $message = $_[0];
    if (@_ > 1) {
      $detail = $_[1];
      if (@_ > 2) {
        $break = $_[2];
        if (@_ > 3) {
          $message = 'Error call has wrong argument number';
        }
      }
    }
  }
  
  my $what = '';

  if ($detail) {
    my @call_0 = caller(0);
    my @call_1 = caller(1);

    my $file_name = $call_0[1];
    my $line = $call_0[2];
    my $function_name = 'main';
    if (@call_1) {
      $function_name = $call_1[3];
    }

    $what .= $file_name.': '.$line.': '.$function_name.'(): ';
  }

  $what .= 'warning';

  if ($message ne '') {
    $what .= ': '.$message;
  }
  $what .= '!';

  if ($break) {
    $what .= "\n";
    $what .= 'Press "Enter" to continue or "Control-C" for breakup...';
    *STDERR->print($what);
    getc(*STDIN);
  }
  else {
    *STDERR->print($what."\n");
  }
}

sub error {
  my $message = '';
  my $detail = 0;

  if (@_ > 0) {
    $message = $_[0];
    if (@_ > 1) {
      $detail = $_[1];
      if (@_ > 2) {
        $message = 'Error call has wrong argument number';
      }
    }
  }
  
  my $what = '';

  if ($detail) {
    my @call_0 = caller(0);
    my @call_1 = caller(1);

    my $file_name = $call_0[1];
    my $line = $call_0[2];
    my $function_name = 'main';
    if (@call_1) {
      $function_name = $call_1[3];
    }

    $what .= $file_name.': '.$line.': '.$function_name.'(): ';
  }

  $what .= 'error';
  if ($message ne '') {
    $what .= ': '.$message;
  }
  $what .= '!';

  *STDERR->print($what."\n");
  exit(1);
}

sub debug {
  my $message = '';
  if (@_ == 1) {
    $message = $_[0];
  }
  elsif (@_ > 1) {
    $message = 'Debug call has wrong argument number';
  }
  
  my @calls = ();
  my $i = 0;
  while (my @caller = caller($i)) {
    push(@calls, \@caller);
    ++$i
  }
  push(@calls, ['', '', 0, 'main']);

  my $what = 'abort';
  if ($message ne '') {
    $what .= ': '.$message;
  }
  $what .= '!';
  *STDERR->print($what."\n");
  for (my $i = 0; $i < @calls - 1; ++$i) {
    my $file_name = $calls[$i]->[1];
    my $line = $calls[$i]->[2];
    my $function_name = $calls[$i + 1]->[3];
    my $location = $file_name.': '.$line.': '.$function_name;
    *STDERR->print('('.$i.') '.$location."\n");
  }
  exit(0);
}

1;
